from enum import Enum

class Direction(Enum):
    NONE = (None, None)
    LEFT = (0, -1)
    UP = (-1, 0)
    RIGHT = (0, 1)
    DOWN = (1, 0)
    ENTER = (0, 0)
    REPLACE = (None, 0)

    def __init__(self, x, y):
        self.x = x
        self.y = y

    def get_loc(self, loc):
        if self.x is None:
            return loc
        return (loc[0] + self.x, loc[1] + self.y)

    @classmethod
    def get_dir(cls, loc1, loc2):
        vector = (loc2[0] - loc1[0], loc2[1] - loc1[1])
        if vector == (0, -1):
            return cls.LEFT
        elif vector == (-1, 0):
            return cls.UP
        elif vector == (0, 1):
            return cls.RIGHT
        elif vector == (1, 0):
            return cls.DOWN
        else:
            return cls.NONE



from .bot import Bot
from ..direction import Direction
from .algs import BFS

class LineFollower(Bot):
    def __init__(self, board, loc, speed, id):
        super().__init__(board, loc, speed, id)
        self.queue = []
        self.visited = set()
    
    def compute_step(self):
        if not self.is_in_line():
            src = self.loc
            self.queue = BFS(self.board, src, 
                             lambda tile: tile.is_end_of_line() \
                             and tile.get_line() not in self.visited)
            if self.queue is None:
                self.visited = set()
                self.queue = BFS(self.board, src, 
                             lambda tile: tile.is_end_of_line() \
                             and tile.get_line() not in self.visited)

        if self.queue:
            if self.loc == self.queue[0]:
                self.queue.pop(0)
            if self.queue:
                first_tile = self.queue[0]
                self.new_direction = Direction.get_dir(self.loc, first_tile)
            else:
                self.new_direction = Direction.ENTER
                self.visited.add(self.board.get(self.loc).get_line())



from enum import Enum
from .direction import Direction

class State(object):
    def __init__(self, bot):
        team_id = bot.get_team_id()
        loc = bot.get_loc()
        self.dir = bot.direction
        self.team_id = -1 if team_id is None else team_id
        self.id = bot.get_id()
        self.x = loc[0]
        self.y = loc[1]
        self.progress = bot.progress
        self.threshold = bot.board.get(self.dir.get_loc(loc)).get_threshold()
        self.line_pos = -1 if bot.line_pos is None else bot.line_pos

        if bot.is_in_line():
            self.state = "inline"
        elif bot.direction == Direction.LEFT:
            self.state = "left"
        elif bot.direction == Direction.RIGHT:
            self.state = "right"
        elif bot.direction == Direction.DOWN:
            self.state = "down"
        elif bot.direction == Direction.UP:
            self.state = "up"
        else:
            self.state = "none"

    def __repr__(self):
        return "ID_%02d @%02d.%02d P_%02d_T_%02d_L_%02d" % (
            self.id, self.x, self.y, self.progress, self.threshold, self.line_pos)

    # self.state is None?
    def get_num_encoding(self):
        return "{} {} {} {} {} {} {} {}".format(self.team_id, self.id, self.x,
                                               self.y, self.state, 
                                               self.progress, self.threshold,
                                               self.line_pos)




from ..direction import Direction

class Bot(object):

    def __init__(self, board, loc, speed, id, team_id=None):
        """Initializes a basic bot."""
        if board:
            (board.get(loc)).add_bot(self)
        else:
            self.loc = loc
        self.board = board
        self.progress = 0
        self.speed = speed
        self.type = None
        self.id = id
        self.team = team_id

        self.direction = Direction.NONE
        self.new_direction = Direction.NONE

        self.line_pos = None # This is set by Line, do not touch
        self.line_name = None
        self.in_line = False

    def __str__(self):
        return "{}: {}".format(self.id, self.loc)

    def __repr__(self):
        return "{}: {}".format(self.id, self.loc)

    def copy(self):
        """Generate a copy of the bot for player copy."""
        new_bot = Bot(None, self.loc, self.speed, self.id)
        new_bot.progress = self.progress
        new_bot.type = self.type

        new_bot.direction = self.direction
        new_bot.new_direction = Direction.NONE # They don't need this information.

        new_bot.in_line = self.in_line

        return new_bot

    def is_in_line(self):
        """Returns true if in line, and false if not in line."""
        return self.in_line

    def set_id(self, new_id):
        self.id = new_id

    def get_id(self):
        return self.id

    def get_team_id(self):
        return self.team

    def set_loc(self, loc):
        """DANGEROUS: do not use except in tile.py"""
        self.loc = loc

    def get_loc(self):
        """Returns the location of the bot."""
        return self.loc

    def set_new_direction(self, dir):
        """Set the direction of this bot if it is valid. Only should be used
        when setting direction specified from the player."""
        if type(dir) != Direction:
            self.new_direction = Direction.NONE
        else:
            self.new_direction = dir

    def compute_step(self):
        """Computes the next step and places it into self.new_direction."""
        raise "You must implement compute_step"

    def update_progress(self):
        """Updates the movement progress of the bot."""
        if self.direction == Direction.NONE:
            self.progress = 0
            return
        elif self.direction == Direction.ENTER:
            self.progress = 0
            self.in_line = self.board.get(self.loc).add_to_line(self)
            self.direction = Direction.NONE
            return

        self.in_line = False
        self.line_pos = None
        self.line_name = None

        self.progress += self.speed
        new_loc = (self.direction).get_loc(self.loc)
        dest_tile = (self.board).get(new_loc)
        if self.progress >= dest_tile.get_threshold():
            curr_tile = (self.board).get(self.loc)
            curr_tile.remove_bot(self)
            dest_tile.add_bot(self)
            self.direction = Direction.NONE
            self.progress = 0

    def execute_step(self):
        """Executes the computed step, if valid."""
        if self.new_direction != self.direction:
            if not self._is_valid(self.new_direction):
                self.new_direction = Direction.NONE
            self.direction = self.new_direction
            self.progress = 0
        self.update_progress()

    def _is_valid(self, direction):
        """Checks if self.new_direction is a valid direction."""
        new_loc = (direction).get_loc(self.loc)
        if (0 <= new_loc[0] < (self.board).x_dim() and
           0 <= new_loc[1] < (self.board).y_dim()):
            dest_tile = (self.board).get(new_loc)
            if not dest_tile.get_booth():
                return True
        return False



class Tile(object):
    """ Tile keeps track of everything on a tile.

    loc: the lattice position of the tile
    bots_in_line: the bots on this tile that are in a line (if a line exists)
    bots: the bots on this tile that are not in a line
    end_of_line: indicates true if this is the current "end of the line";
                 that is, bots enter the line on this tile.
    line: None if not part of a line, otherwise a Line object
    booth: None if free, otherwise a Booth object
    threshold: progress needed to move into this tile
    """
    def __init__(self, x, y):
        self.loc = (x, y)
        self.bots_in_line = []
        self.bots = []
        self.end_of_line = False
        self.line = None
        self.booth = None
        self.update_threshold()

        self.visible = True

    def __str__(self):
        return "({}, {})".format(len(self.bots), len(self.bots_in_line))

    def __repr__(self):
        if not self.booth == None:
            return "BOOTH"
        if not self.visible:
            return "TXXXX"
        if self.end_of_line:
            return "E%02d%02d" % (len(self.bots), len(self.bots_in_line))
        if not self.line == None:
            return "L%02d%02d" % (len(self.bots), len(self.bots_in_line))
        return "T%02d%02d" % (len(self.bots), len(self.bots_in_line))
        #return "({}, {})".format(len(self.bots), len(self.bots_in_line))

    # Getters that are available to players.
    def is_visible(self):
        return self.visible

    def get_line(self):
        """
        If this tile is part of a line, returns the name of the company
        it is a line for. Otherwise, returns None."""
        return self.line

    def get_booth(self):
        """
        If this tile is part of a booth, returns the name of the company
        of the booth. Otherwise, returns None."""
        return self.booth

    def get_num_bots(self):
        """
        Returns the number of bots on this tile.
        
        If the tile is part of a line, the number of bots on the tile
        includes the number of bots in the line."""
        return len(self.bots) + len(self.bots_in_line)

    def is_end_of_line(self):
        """
        If the current end of the line is on this tile, returns True.
        Otherwise, returns False.
        
        Note that this method returns False even if this line is not
        part of a tile (tile.get_line() == None)."""
        return self.end_of_line

    # Setters used only in initialization
    def set_line(self, name):
        self.line = name

    def set_booth(self, name):
        self.booth = name

    # Functions available to non-players
    def get_threshold(self):
        return self.threshold

    def get_loc(self):
        """Returns the location of this tile."""
        return self.loc

    # Functions that will only be available to the internal game mechanics.
    def update_threshold(self):
        pop = self.get_num_bots()
        if pop == 0:
            self.threshold = 1
        elif pop == 1:
            self.threshold = 1
        elif pop == 2:
            self.threshold = 1
        elif pop == 3:
            self.threshold = 1
        elif pop == 4:
            self.threshold = 2
        elif pop == 5:
            self.threshold = 2
        elif pop == 6:
            self.threshold = 2
        elif pop == 7:
            self.threshold = 3
        elif pop == 8:
            self.threshold = 3
        else:
            self.threshold = 5

    def get_bots_in_line(self):
        return self.bots_in_line

    def set_end_of_line(self, value):
        self.end_of_line = value

    def update_line(self, line):
        self.bots_in_line = line
        for bot in line:
            bot.set_loc(self.loc)

    def add_bot(self, bot):
        (self.bots).append(bot)
        bot.set_loc(self.loc)

    def remove_bot(self, bot):
        try:
            (self.bots).remove(bot)
        except:
            (self.bots_in_line).remove(bot)

    def add_to_line(self, bot):
        if self.end_of_line and bot in self.bots:
            (self.bots_in_line).append(bot)
            self.remove_bot(bot)
            return True
        return False

    def copy(self):
        new_tile = Tile(self.loc[0], self.loc[1])
        new_tile.bots = [bot.copy() for bot in self.bots]
        new_tile.bots_in_line = [bot.copy() for bot in self.bots_in_line]
        new_tile.booth = self.booth
        new_tile.line = self.line
        new_tile.end_of_line = self.end_of_line
        new_tile.update_threshold()

        new_tile.visible = self.visible

        return new_tile

class Booth(object):
    def __init__(self, name, size, booth_tiles, line_tiles, wait_time, value):
        """Booth handles the set of tiles that constitute a company's booth.
        It initializes the lines and stores them.

        name: the name of the booth (that is, the name of the company)
        size: size of the company for the booth.
        booth_tiles: list of tiles that constitute the booth
        line_tiles: list of tiles which start with the tile that is
                    the end of the line.
        wait_time: the time it takes the recruiter to talk to the person
                   at the front of the line.
        """
        self.name = name
        self.size = size
        self.tiles = booth_tiles
        for tile in booth_tiles:
            tile.set_booth(name)
        self.line = Line(self.name, line_tiles, wait_time)
        self.value = value

    def get_size(self):
        """Gets the size of the company."""
        return self.size

    def execute_step(self):
        """Executes steps for the line."""
        return (self.line).execute_step()
    
    def get_end(self):
        return self.line.get_end()

class Line(object):
    """
    Line is a collection of tiles that are part of the "line zone".
    Line is responsible for moving the bots in a line if they choose
    to stay in the line.
    """
    def __init__(self, name, tiles, wait_time, max_per_tile=3):
        """
        name: Name of the company the line is for.
        tiles: The list of tiles that constitute the line. The first tile
               in the list should be
        wait_time: The number of turns it takes for the recruiter to talk
                   to the person at the front of the line.
        max_per_tile: The max number of people per tile.
        progress: The current progress of the recruiter.
        current_talker: the bot that is currently talking to the recruiter.
        """
        self.name = name
        for tile in tiles:
            tile.set_line(name)
        self._order(tiles)

        self.wait_time = wait_time
        self.progress = 0
        self.current_talker = None
        self.max_per_tile = max_per_tile

        self.end_of_line = tiles[0]
        self.tiles[0].set_end_of_line(True)

    def execute_step(self):
        """Compresses the line, updates the talker, and redelegates lines."""
        ret_val = None

        full_line = []
        for tile in self.tiles:
            full_line += tile.get_bots_in_line()
        if full_line:
            front_of_line = full_line[0]
            if front_of_line == self.current_talker:
                self.progress += 1
                if self.progress >= self.wait_time:
                    full_line.pop(0)
                    self.current_talker.in_line = False
                    self.current_talker.line_pos = None
                    self.current_talker.line_name = None
                    self.progress = 0
                    self.tiles[0].add_bot(self.current_talker)

                    team_id = self.current_talker.get_team_id()
                    if team_id is not None:
                        bot_id = self.current_talker.get_id()
                        ret_val = (self.name, team_id, bot_id)
            else:
                self.progress = 0
                self.current_talker = front_of_line
        else:
            self.current_talker = None
        for i, bot in enumerate(full_line):
            bot.in_line = True
            bot.line_pos = i
            bot.line_name = self.name

        self._delegate(full_line)
        return ret_val

    def _delegate(self, line):
        """Delegates the bots into the lines."""
        last_tile = self.tiles[-1]
        end = False
        for tile in self.tiles:
            if tile == last_tile:
                tile.update_line(line)
                if not end:
                    tile.set_end_of_line(True)
                else:
                    tile.set_end_of_line(False)
            else:
                mini_line = line[:self.max_per_tile]
                line = line[self.max_per_tile:]
                if len(mini_line) < self.max_per_tile and (not end):
                    end = True
                    self.end_of_line = tile
                    tile.set_end_of_line(True)
                else:
                    tile.set_end_of_line(False)
                tile.update_line(mini_line)

    def _order(self, tiles):
        if len(tiles) <= 1:
            self.tiles = tiles
        else:
            sort_index = None
            rev = False

            end_loc = tiles[0].get_loc()
            comp = tiles[1].get_loc()
            if end_loc[0] == comp[0]:
                sort_index = 1
            else:
                sort_index = 0
            if end_loc[sort_index] < comp[sort_index]:
                rev = True

            self.tiles = sorted(tiles,
                                key=lambda x: x.loc[sort_index],
                                reverse=rev)

    def get_end(self):
        """Returns the tile that is the end of the line."""
        return self.end_of_line



# THINGS TO DO

# Write proper threshold formula and add working crowd (make bots)

# Check and debug code!!


from .tile import Tile, Booth, Line
from .bots import *
from .state import State
from .direction import Direction
import numpy as np

import random

class Board(object):

    wait_time = 1
    major_reward = 10
    minor_reward = 5
    visible_range = 2

    # add constants regarding main player + min player scores?

    """
    Sample configuration file:
    6 8 6
    F  M3 M3 M3 M4 M4 M4 F
    S0  0 E0  3 E5  4  F F
    S0 E1 E2  3  5  4  F F
    S1  1  2  3  5  4  F B
    S1  F  2 E3  5 E4  F F
    F  S2 S2 L5 L5 L5 L5 F
    """
    def __init__(self, config_file, company_file, log_file, debug, team_size, num_players):
        """Initializes the Board class.

        grid: Grid of tiles
        booths: The booth objects on the grid.
        players: The number of players playing on this board instance.
        player_bots: The bots associated with each player. Each element is
                     a list of 4 bots that correlate to its index's player #.
        dim: The dimensions of the board.
        """
        self.time_step = 0
        self.log_file = log_file
        self.team_size = team_size

        self.grid = []
        self.booths = dict()
        self.dim = (None, None)
        self.company_list = {'S': set(), 'M': set(), 'L': set()}
        self.chosen_companies = set()

        self.player_bots = []
        self.bots = []
        self.players = num_players
        self.start = None

        self.company_values = {}
        self.company_info = {}

        self._parse_companies(company_file)
        self._parse(config_file)

        self.debug = debug

        # 2 players at most
        self.scores = [0] * self.players
        self.rewards = [self.company_values.copy() for _ in range(self.players)]

    def dbg_print(self, str):
        if self.debug:
            print(str)

    def init_bots(self):
        """Places the bots onto the board."""
        # Do the initialization of multiple bots/place them!
        for i in range(self.players):
            # First bot in this list is main bot by standard.
            team_i = []
            for j in range(self.team_size):
                team_i.append(Bot(self, self.start, 1, j, team_id=i))
            (self.player_bots).append(team_i)

        # TODO: Initialize the crowd!

        start_id = 0
        for i in range(0, self.num_bots//10):
            num = i * 10
            self.bots.append(JitteryBot(self, self.start, 1, num))
            num += 1
            self.bots.append(JitteryBot(self, self.start, 1, num))
            num += 1
            self.bots.append(JitteryBot(self, self.start, 1, num))
            num += 1
            self.bots.append(ExtrovertBot(self, self.start, 1, num))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num, "S"))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num, "M"))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num, "L"))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num, "L"))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num))
            num += 1
            self.bots.append(RandomBot(self, self.start, 1, num))
        #for i in range(start_id, start_id + 5):
            #self.bots.append(JitteryBot(self, self.start, 1, i))

        self._log('w')

    def x_dim(self):
        return self.dim[0]

    def y_dim(self):
        return self.dim[1]

    def get(self, loc):
        """Gets the tile at loc."""
        if 0 <= loc[0] < self.dim[0] and 0 <= loc[1] < self.dim[1]:
            return self.grid[loc[0]][loc[1]]
        else:
            return None

    # Note: you can only replace a main bot with a helper bot.
    # (Replacing a helper with a helper is functionally useless.)
    def step(self, team_moves):
        """Updates the whole board by one step."""
        for bot in self.bots:
            bot.compute_step()
        for team, moves in zip(self.player_bots, team_moves):
            if moves[0] == Direction.REPLACE:
                self.dbg_print("Replacing!")
                swap_index = -1
                invalid = False

                main_loc = team[0].get_loc()
                for i in range(1, self.team_size):
                    bot_loc = team[i].get_loc()
                    if moves[i] == Direction.REPLACE and bot_loc == main_loc:
                        if swap_index == -1:
                            self.dbg_print("Found the bot to replace with.")
                            swap_index = i
                        else:
                            self.dbg_print("Found another one; replace is invalid.")
                            moves[i] = Direction.NONE
                            invalid = True
                    elif moves[i] == Direction.REPLACE:
                        self.dbg_print("Found a replace not on the same location.")
                        moves[i] = Direction.NONE

                moves[0] = Direction.NONE
                if swap_index != -1:
                    moves[swap_index] = Direction.NONE
                    if not invalid:
                        self._swap(team, swap_index)
            self.dbg_print("Team: {}".format(team))
            self.dbg_print("New moves: {}".format(moves))

            for bot, move in zip(team, moves):
                bot.set_new_direction(move)

        for bot in self.bots:
            bot.execute_step()

        for team in self.player_bots:
            for bot in team:
                bot.execute_step()

        talked_booths = []
        for name in self.booths:
            booth = self.booths[name]
            val = booth.execute_step()
            if val:
                talked_booths.append(val)
        updated_scores = self._score(talked_booths)

        self.dbg_print(np.matrix(self.grid))
        self._update_board()
        self._log('a+')

        self.scores = updated_scores

        return updated_scores

    def _swap(self, arr, index):
        """Swaps bot at index 0 with bot at index index in arr."""
        arr[0], arr[index] = arr[index], arr[0]

        temp_id = arr[0].get_id()
        arr[0].set_id(arr[index].get_id())
        arr[index].set_id(temp_id)

    def _score(self, talked):
        scores = [0] * self.players
        for name, team_id, id in talked:
            # Change these values to add prioritization for
            # certain companies!
            reward_dict = self.rewards[team_id]
            (major, minor) = reward_dict[name]
            if id == 0:
                scores[team_id] += major
            else:
                scores[team_id] += minor
            reward_dict[name] = (major//2, minor//2)
        return scores

    def get_visible_locs(self, team):
        """Gets the visible locations for the specified team."""
        visible_locs = set()
        team_bots = self.player_bots[team]
        for bot in team_bots:
            x, y = bot.get_loc()
            for i in range(x-self.visible_range, x+self.visible_range+1):
                for j in range(y-self.visible_range, y+self.visible_range+1):
                    if 0 <= i < self.dim[0] and 0 <= j < self.dim[1]:
                        visible_locs.add((i, j))
        return list(visible_locs)

    def get_states(self, team):
        """Gets the states of the bots in the team."""
        team_bots = self.player_bots[team]
        return [State(bot) for bot in team_bots]

    def _update_board(self):
        """Updates the thresholds."""
        for row in self.grid:
            for tile in row:
                tile.update_threshold()

    def _parse_companies(self, file_path):
        """Parses the company file."""
        file = open(file_path, "r")
        for line in file:
            split_line = line.split()
            company, size, value = split_line[0], split_line[1], int(split_line[2])
            self.company_list[size].add((company, value))

    def _pick_company(self, size):
        """Picks a company from the list of all companies."""
        companies = self.company_list[size]
        if len(companies) <= 0:
            raise Exception("Not enough companies for {}".format(size))
        chosen = random.sample(companies, 1)[0]
        companies.remove(chosen)
        self.chosen_companies.add(chosen)
        return chosen

    def _random_time(self, size):
        """Picks a random time. Currently 5, will change if we decide to add
        random times."""
        return Board.wait_time

    def _parse(self, file_path):
        """Parses a config file into a grid of tiles and booths."""
        file = open(file_path, "r")

        first_line = True
        r = 0
        sizes = []
        booth_tiles = []
        line_tiles = []

        for line in file:
            if first_line:
                dims = line.split()
                self.dim = (int(dims[0]), int(dims[1]))
                self.grid = np.empty(self.dim, dtype=object)
                for i in range(self.dim[0]):
                    #row = []
                    for j in range(self.dim[1]):
                        self.grid[i][j] = Tile(i, j)
                        #row.append(Tile(i, j))
                    #(self.grid).append(row)
                num_companies = int(dims[2])
                self.num_bots = int(dims[3])
                self.visible_range = int(dims[4])
                sizes = [None] * num_companies
                booth_tiles = [[] for _ in range(num_companies)]
                line_tiles = [[] for _ in range(num_companies)]
                first_line = False
            else:
                c = 0
                tiles = line.split()
                for tile in tiles:
                    letter = tile[0]
                    if letter == 'S' or letter == 'M' or letter == 'L':
                        number = int(tile[1:])
                        if not sizes[number]:
                            sizes[number] = letter
                        curr_booth = booth_tiles[number]
                        curr_booth.append(self.grid[r][c])
                    elif letter == 'E':
                        number = int(tile[1:])
                        curr_line = line_tiles[number]
                        curr_line.insert(0, self.grid[r][c])
                    elif letter == 'B':
                        self.start = (r, c)
                    elif letter != 'F':
                        number = int(tile)
                        curr_line = line_tiles[number]
                        curr_line.append(self.grid[r][c])
                    c += 1
                r += 1

        #print(len(list(filter(lambda x: x == "S", sizes))))
        #print(len(list(filter(lambda x: x == "M", sizes))))
        #print(len(list(filter(lambda x: x == "L", sizes))))

        for size, b_tiles, l_tiles in zip(sizes, booth_tiles, line_tiles):
            name, value = self._pick_company(size)
            self.company_info[name] = value
            self.company_values[name] = (value, value//2)
            time = self._random_time(size)
            (self.booths)[name] = Booth(name, size, b_tiles, l_tiles, time, value)

    def _log(self, mode):
        with open(self.log_file, mode) as log:
            if mode == 'w':
                team2 = self.team_size if self.players == 2 else 0
                log.write("{} {} {}\n".format(self.team_size, team2,
                                              len(self.bots)))
                for company in self.chosen_companies:
                    log.write("{}\n".format(company[0]))
                log.write("\n")

            log.write("{} {} {}\n".format(self.time_step, self.scores[0], self.scores[1] if self.players > 1 else 0))
            for team in self.player_bots:
                for bot in team:
                    log.write("{}\n".format(State(bot).get_num_encoding()))
            for bot in self.bots:
                log.write("{}\n".format(State(bot).get_num_encoding()))
        self.time_step += 1



from .board import Board

class Game(object):
    def __init__(self, config_file, companies, log_file, multiplayer=False, debug=False, team_size=4):
        """Initialize a game instance."""
        self.players = 2 if multiplayer else 1
        self.board = Board(config_file, companies, log_file, debug, team_size,
            2 if multiplayer else 1)
        self._copy()
        (self.board).init_bots()

        self.scoreboard = [0] * self.players

    def get_companyinfo(self):
        return self.board.company_info

    def _copy(self):
        """Copies the given board into a basic grid for player use."""
        grid = self.board.grid
        new_grid = []
        for row in grid:
            new_row = []
            for tile in row:
                new_tile = tile.copy()
                new_tile.end_of_line = False
                new_tile.visible = False
                new_row.append(new_tile)
            new_grid.append(new_row)
        self.basic_grid = new_grid

    def generate_player_copy(self, team=0, init=False):
        """Uses the basic grid and the visible locations to generate a new
        grid with tiles from basic grid if not visible and tiles from the
        board if visible. Should use copied tiles if visible."""
        visible_locs = (self.board).get_visible_locs(team)
        player_copy = []
        for row in self.board.grid:
            new_row = []
            for tile in row:
                loc = tile.get_loc()
                if loc in visible_locs and not init:
                    new_row.append(tile.copy())
                else:
                    obj = (self.basic_grid[loc[0]][loc[1]]).copy()
                    new_row.append(obj)
            player_copy.append(new_row)
        return player_copy

    def make_move(self, moves):
        """Update for multiplayer use later."""
        updated_scores = (self.board).step(moves)
        for i, score in enumerate(updated_scores):
            self.scoreboard[i] += score

        return [(self.generate_player_copy(i), (self.board).get_states(i),
                self.scoreboard[i]) for i in range(self.players)]



from setuptools import setup, find_packages

setup(name='awap2019',
      version='2.0',
      description='Module for AWAP 2019.',
      long_description='Contains essential classes for competitors.',
      url='http://github.com/cuebeomc/awap2019',
      author='ACM@CMU',
      author_email='cuebeomc@andrew.cmu.edu',
      license='GPL',
      packages=find_packages(),
      install_requires=[
          'numpy',
          'absl-py',
          'matplotlib',
          'scipy'
      ],
      )



from .bot import Bot
from ..direction import Direction
from .algs import BFS

class StillBot(Bot):
    """Simple class to test BFS"""
    def __init__(self, board, loc, speed, id):
        super().__init__(board, loc, speed, id)
        self.move = Direction.NONE
    
    def compute_step(self):
        src = self.loc
        dest = (4, 1)
        x = BFS(self.board, src, lambda tile: tile.get_loc() == dest)
        print("BFS list: {}".format(x))
        return self.move

